### 杭州电子科技大学 小型内核实现方向
### T20241033699435 "8C乡交"
#### 队伍成员：董新心 陈春夏 指导老师：刘真 马虹
####    完成情况
       我们主要工作是增加了6个系统调用，丰富了xv6的系统功能。所有这些功能都是通过扩展xv6的系统调用来实现的。系统调用是用户程序与操作系统内核交互的接口，这些扩展提供了新的接口，使得用户程序能够获取更多关于系统状态的信息。
      sys_get_cpu_usage、sys_totalmem这些系统调用都与系统资源的监控和管理有关。它们允许用户程序获取关于CPU使用率、进程运行时间、系统总内存和空闲内存的信息，这些都是系统资源管理的关键指标。
      sys_cps系统调用与进程管理直接相关，它允许用户查看系统中所有进程的状态和优先级。这与sys_get_cpu_usage和sys_gettime相辅相成，因为了解进程的状态和优先级对于分析CPU使用率和运行时间至关重要。
      这些扩展都是为了增强xv6作为一个教学操作系统的功能。通过实现这些系统调用，学生可以更深入地理解操作系统的工作原理，包括进程管理、内存管理和系统监控等。

### xv6内核实现-系统调用
### sys_get_cpu_usage: 获取当前进程的CPU使用率
### 功能描述
`sys_get_cpu_usage` 系统调用允许用户获取当前进程的CPU使用率。此功能对于性能分析和进程管理非常有用。

### 实现步骤

1. **修改 `proc.c` 实现系统调用功能**

   ```c
   // proc.c
   int get_cpu_usage(void) {
       struct proc *p = myproc();
       return p->cpu_ticks * 100 / ticks;
   }
   ```

2. **在 `user.h` 和 `defs.h` 中定义系统调用函数**

   ```c
   // user.h
   int get_cpu_usage(void);

   // defs.h
   int get_cpu_usage(void);
   ```

3. **在 `usys.S` 中添加系统调用接口**

   ```asm
   // usys.S
   SYSCALL(get_cpu_usage)
   ```

4. **在 `syscall.h` 中定义系统调用号**

   ```c
   // syscall.h
   #define SYS_get_cpu_usage 27
   ```

5. **在 `syscall.c` 中注册系统调用函数**

   ```c
   // syscall.c
   extern int sys_get_cpu_usage(void);
   static int (*syscalls[])(void) = {
       ...
       [SYS_get_cpu_usage] sys_get_cpu_usage,
       ...
   };
   ```

6. **在 `sysproc.c` 中实现系统调用的分发**

   ```c
   // sysproc.c
   int sys_get_cpu_usage(void) {
       return get_cpu_usage();
   }
   ```

7. **创建用户程序 `get_cpu_usage.c`**

   ```c
   // get_cpu_usage.c
   #include "types.h"
   #include "user.h"

   int main(void) {
       int cpu_usage = get_cpu_usage();
       printf(1, "CPU usage: %d%%\n", cpu_usage);
       exit();
   }
   ```

8. **修改 `Makefile` 以编译新的用户程序**

   ```makefile
   # Makefile
   UPROGS=... get_cpu_usage

   ...

   get_cpu_usage: get_cpu_usage.o
       $(LD) $(LDFLAGS) -o $@ get_cpu_usage.o -lc
   ```

9. **编译和测试**

   ```bash
   make clean
   make
   make qemu
   ```

   在xv6的shell中运行新的用户程序 `get_cpu_usage`：

   ```bash
   get_cpu_usage
   ``

## sys_gettime: 获取当前进程的运行时间

### 功能描述
`sys_gettime` 系统调用允许用户获取当前进程的运行时间，以时钟周期为单位。

### 实现步骤

1. **修改 `proc.c` 实现系统调用功能**

   ```c
   // proc.c
   int gettime(void) {
       struct proc *p = myproc();
       return p->runtime;
   }
   ```

2. **在 `user.h` 和 `defs.h` 中定义系统调用函数**

   ```c
   // user.h
   int gettime(void);

   // defs.h
   int gettime(void);
   ```

3. **在 `usys.S` 中添加系统调用接口**

   ```asm
   // usys.S
   SYSCALL(gettime)
   ```

4. **在 `syscall.h` 中定义系统调用号**

   ```c
   // syscall.h
   #define SYS_gettime 26
   ```

5. **在 `syscall.c` 中注册系统调用函数**

   ```c
   // syscall.c
   extern int sys_gettime(void);
   static int (*syscalls[])(void) = {
       ...
       [SYS_gettime] sys_gettime,
       ...
   };
   ```

6. **在 `sysproc.c` 中实现系统调用的分发**

   ```c
   // sysproc.c
   int sys_gettime(void) {
       return gettime();
   }
   ```

7. **创建用户程序 `gettime.c`**

   ```c
   // gettime.c
   #include "types.h"
   #include "user.h"

   int main(void) {
       int runtime = gettime();
       printf(1, "Process runtime: %d ticks\n", runtime);
       exit();
   }
   ```

8. **修改 `Makefile` 以编译新的用户程序**

   ```makefile
   # Makefile
   UPROGS=... gettime

   ...

   gettime: gettime.o
       $(LD) $(LDFLAGS) -o $@ gettime.o -lc
   ```

9. **编译和测试**

   ```bash
   make clean
   make
   make qemu
   ```

   在xv6的shell中运行新的用户程序 `gettime`：

   ```bash
   gettime
   ```
## sys_totalmem: 获取当前系统的总内存大小

### 功能描述
`sys_totalmem` 系统调用允许用户获取当前系统的总内存大小。

### 实现步骤

1. **修改 `proc.c` 实现系统调用功能**

   ```c
   // proc.c
   int totalmem(void) {
       return PHYSTOP;
   }
   ```

2. **在 `user.h` 和 `defs.h` 中定义系统调用函数**

   ```c
   // user.h
   int totalmem(void);

   // defs.h
   int totalmem(void);
   ```

3. **在 `usys.S` 中添加系统调用接口**

   ```asm
   // usys.S
   SYSCALL(totalmem)
   ```

4. **在 `syscall.h` 中定义系统调用号**

   ```c
   // syscall.h
   #define SYS_totalmem 24
   ```

5. **在 `syscall.c` 中注册系统调用函数**

   ```c
   // syscall.c
   extern int sys_totalmem(void);
   static int (*syscalls[])(void) = {
       ...
       [SYS_totalmem] sys_totalmem,
       ...
   };
   ```

6. **在 `sysproc.c` 中实现系统调用的分发**

   ```c
   // sysproc.c
   int sys_totalmem(void) {
       return totalmem();
   }
   ```

7. **创建用户程序 `totalmem.c`**

   ```c
   // totalmem.c
   #include "types.h"
   #include "user.h"

   int main(void) {
       int total_memory = totalmem();
       printf(1, "Total memory: %d bytes\n", total_memory);
       exit();
   }
   ```

8. **修改 `Makefile` 以编译新的用户程序**

   ```makefile
   # Makefile
   UPROGS=... totalmem

   ...

   totalmem: totalmem.o
       $(LD) $(LDFLAGS) -o $@ totalmem.o -lc
   ```

9. **编译和测试**

   ```bash
   make clean
   make
   make qemu
   ```

   在xv6的shell中运行新的用户程序 `totalmem`：

   ```bash
   totalmem
   ```


## sys_freemem: 获取当前系统的空闲内存大小

### 功能描述
`sys_freemem` 系统调用允许用户获取当前系统的空闲内存大小。

### 实现步骤

1. **修改 `proc.c` 实现系统调用功能**

   ```c
   // proc.c
   int freemem(void) {
       uint free_pages = nfree();
       return free_pages * PGSIZE;
   }
   ```

2. **在 `user.h` 和 `defs.h` 中定义系统调用函数**

   ```c
   // user.h
   int freemem(void);

   // defs.h
   int freemem(void);
   ```

3. **在 `usys.S` 中添加系统调用接口**

   ```asm
   // usys.S
   SYSCALL(freemem)
   ```

4. **在 `syscall.h` 中定义系统调用号**

   ```c
   // syscall.h
   #define SYS_freemem 23
   ```

5. **在 `syscall.c` 中注册系统调用函数**

   ```c
   // syscall.c
   extern int sys_freemem(void);
   static int (*syscalls[])(void) = {
       ...
       [SYS_freemem] sys_freemem,
       ...
   };
   ```

6. **在 `sysproc.c` 中实现系统调用的分发**

   ```c
   // sysproc.c
   int sys_freemem(void) {
       return freemem();
   }
   ```

7. **创建用户程序 `freemem.c`**

   ```c
   // freemem.c
   #include "types.h"
   #include "user.h"

   int main(void) {
       int free_memory = freemem();
       printf(1, "Free memory: %d bytes\n", free_memory);
       exit();
   }
   ```

8. **修改 `Makefile` 以编译新的用户程序**

   ```makefile
   # Makefile
   UPROGS=... freemem

   ...

   free
   mem: freemem.o
       $(LD) $(LDFLAGS) -o $@ freemem.o -lc
   ```

9. **编译和测试**

   ```bash
   make clean
   make
   make qemu
   ```

   在xv6的shell中运行新的用户程序 `freemem`：

   ```bash
   freemem
   ```

## sys_hello: 打印 "图案+Hello, xv6!" 到控制台

### 功能描述
`sys_hello` 系统调用允许用户打印一条消息 "Hello, xv6!" 到控制台。

### 实现步骤

1. **实现系统调用功能**

   ```c
   // sysproc.c
   #include "types.h"
   #include "x86.h"
   #include "defs.h"
   #include "param.h"
   #include "memlayout.h"
   #include "mmu.h"
   #include "proc.h"

   
   int
   sys_hello(void){
   cprintf("   ___ \n");
   cprintf("  / _ \\n");
   cprintf(" | (_) |__   __ _ _ __   __ _ \n");
   cprintf("  \\__, '_ \\ / _` | '_ \\ / _` |\n");
   cprintf("  |___/ |__/ \\__,_| .__/ \\__, |\n");
   cprintf("                |___/ \n");
   cprintf("Hello, xv6!\n");
   return 0;}
    
   ```

2. **在 `user.h` 和 `defs.h` 中定义系统调用函数**

   ```c
   // user.h
   int hello(void);

   // defs.h
   int hello(void);
   ```

3. **在 `usys.S` 中添加系统调用接口**

   ```asm
   // usys.S
   SYSCALL(hello)
   ```

4. **在 `syscall.h` 中定义系统调用号**

   ```c
   // syscall.h
   #define SYS_hello 23
   ```

5. **在 `syscall.c` 中注册系统调用函数**

   ```c
   // syscall.c
   extern int sys_hello(void);
   static int (*syscalls[])(void) = {
       ...
       [SYS_hello]     sys_hello,
       ...
   };
   ```

6. **在 `sysproc.c` 中实现系统调用的分发**

   ```c
   // sysproc.c
   ...
   switch(syscall_number) {
       ...
       case SYS_hello:
           ret = sys_hello();
           break;
       ...
   }
   ...
   ```

7. **创建用户程序 `hello.c`**

   ```c
   // hello.c
   #include "types.h"
   #include "user.h"

   int main(void) {
       hello();
       exit();
   }
   ```

8. **修改 `Makefile` 以编译新的用户程序**

   ```makefile
   # Makefile
   UPROGS=... hello

   ...

   hello: hello.o
       $(LD) $(LDFLAGS) -o $@ hello.o -lc
   ```

9. **编译和测试**

   ```bash
   make clean
   make
   make qemu
   ```

   在xv6的shell中运行新的用户程序 `hello`：

   ```bash
   hello
   ```

   您应该看到控制台打印出 "Hello, xv6!" 的消息。

 
   
### 功能描述
`sys_cps` 系统调用允许用户查看进程，将进程名，pid和优先级打印出来。

### 实现步骤
1. **在`proc.h`中引入priority表示优先级**
    ```c
   // proc.c
      struct proc {
       ...
         int priority;		           // (0-20)  **新增部分**
     };
   ```

1. **修改 `proc.c` 实现系统调用功能**

   ```c
   // proc.c
    found:
    p->state = EMBRYO;
    p->pid = nextpid++;
    p->priority = 10;  **新增部分**
   ```
   ```c
   //proc.c
   int cps(void)
   {
       struct proc *p; //定义一个结构体(进程控制块)
       sti();	// 中断
       acquire(&ptable.lock); //加锁
       cprintf("name \t pid \t state \t \t priority \n"); //罗列所有的pid
       for(p = ptable.proc; p < &ptable.proc[NPROC]; p++)  //NPROC为64
       {
         if(p->state == SLEEPING) //睡眠
         cprintf("%s \t %d \t SLEEPING \t %d\n", p->name, p->pid, p->priority);
         else if(p->state == RUNNING) //正在执行
         cprintf("%s \t %d \t RUNNING \t %d\n", p->name, p->pid, p->priority);
         else if(p->state == RUNNABLE) //可运行队列
         cprintf("%s \t %d \t RUNNABLE \t %d\n", p->name, p->pid, p->priority);
       }
       release(&ptable.lock); //释放锁
       return 22; //返回22
   }
   ```

2. **在 `user.h` 和 `defs.h` 中定义系统调用函数**

   ```c
   //user.h
   int cps(void);
   //defs.h
   int cps(void);
     ```


3. **在 `usys.S` 中添加系统调用接口**

   ```c
   //usys.S
   SYSCALL(cps)
   ```

4. **在 `syscall.h` 中定义系统调用号**

   ```c
   // syscall.h
   #define SYS_cps 22
   ```

5. **在 `syscall.c` 中注册系统调用函数**

   ```c
   // syscall.c
   extern int sys_cps(void);

   [SYS_cps]     sys_cps,
   ```

6. **在 `sysproc.c` 中实现系统调用的分发**

   ```c
   // sysproc.c
    int sys_cps(void) 
      {
       return cps();
      }

   ```

7. **创建用户程序 `zrzps.c`**

   ```c
   //zrzps.c
      #include "types.h"
      #include "stat.h"
      #include "user.h"
 
     int main(int argc, char *argv[])
        {
	         if(argc!=1)
	       	printf(1, "Usage: zrzps\n");
		      else
		      cps();
		      exit();
	      }

   ```

8.  **修改 `Makefile` 以编译新的用户程序**

   ```makefile
   # Makefile
     _zrzps/
   ```

9.  **编译和测试**

   ```bash
   make clean
   make
   make qemu
   ```

   在xv6的shell中运行新的用户程序 `zrzps`：

   ```bash
   zrzps
   ```
